#include "CSoundManager.h"
#include "CInputManager.h"
#include "CGameManager.h"

#ifndef SAFE_RELEASE
#define SAFE_RELEASE(p)      { if (p) { (p)->Release(); (p)=NULL; } }
#endif

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		XACTNotificationCallback
//   Last Modified:	01/28/2009
//   Description:	Callback to handle XACT Notification messages
//
//////////////////////////////////////////////////////////////////////////////////

void WINAPI XACTNotificationCallback( const XACT_NOTIFICATION* pNotification )
{
	CSoundManager::GetInstance().XACTNotifyCallback(pNotification);
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		GetInstance
//   Last Modified:	01/28/2009
//   Description:	Returns an instance of the Sound Manager
//
//////////////////////////////////////////////////////////////////////////////////

CSoundManager &CSoundManager::GetInstance()
{
	static CSoundManager instance;

	return instance;
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		CSoundManager
//   Last Modified:	01/28/2009
//   Description:	Constructor for Sound Manager
//
//////////////////////////////////////////////////////////////////////////////////

CSoundManager::CSoundManager()
{
	m_bInitialized = false;
	m_bAuditionMode = false;
	m_bDebugMode = false;

	m_bStreamingWaveBankPrepared = false;
	m_bZeroLatencyCueStop = false;
	m_bSongStopped = false;

	m_fMusicVolume = 1.0;
	m_fFXVolume = 1.0;

	m_iBackgroundMusic = -1;
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		~CSoundManager
//   Last Modified:	01/28/2009
//   Description:	Deconstructor for Sound Manager
//
//////////////////////////////////////////////////////////////////////////////////

CSoundManager::~CSoundManager()
{
//	ShutDown();
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		Initialize
//   Last Modified:	01/28/2009
//   Description:	Initialize the XACT Engine and the Sound Manager
//
//////////////////////////////////////////////////////////////////////////////////

HRESULT CSoundManager::Initialize(bool AuditionMode, bool DebugMode)
{
	HRESULT hr;
	HANDLE hFile;
	HANDLE hMapFile;
	DWORD dwFileSize;
	DWORD dwBytesRead;

	m_bAuditionMode = AuditionMode;
	m_bDebugMode = DebugMode;

	hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);

	// Create the Engine
	DWORD dwCreationFlags = 0;	// use creation flags to change init settings

	if( m_bAuditionMode )
		dwCreationFlags |= XACT_FLAG_API_AUDITION_MODE;
	if( m_bDebugMode )
		dwCreationFlags |= XACT_FLAG_API_DEBUG_MODE;

	hr = XACT3CreateEngine(dwCreationFlags, &m_pXACT3Engine);

	if( FAILED(hr) || m_pXACT3Engine == NULL)
		return E_FAIL;

	// Load the Global Settings
	VOID* pGlobalSettingsData = NULL;
	DWORD dwGlobalSettingsFileSize = 0;
	bool bSuccess = false;

	hFile = CreateFile( "Resources/Audio/Win/RoidRacer.xgs", GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL );

	if( hFile )
	{
		dwGlobalSettingsFileSize = GetFileSize( hFile, NULL );
		if( dwGlobalSettingsFileSize != INVALID_FILE_SIZE )
		{
			pGlobalSettingsData = CoTaskMemAlloc( dwGlobalSettingsFileSize );
			if( pGlobalSettingsData )
			{
				if( 0 != ReadFile( hFile, pGlobalSettingsData, dwGlobalSettingsFileSize, &dwBytesRead, NULL ) )
				{
					bSuccess = true;
				}
			}
		}
		CloseHandle( hFile );
	}

	if( !bSuccess )
	{
		if( pGlobalSettingsData )
			CoTaskMemFree( pGlobalSettingsData );
		pGlobalSettingsData = NULL;
		dwGlobalSettingsFileSize = 0;
	}

	// Default parameters
	XACT_RUNTIME_PARAMETERS EngineParameters = {0};
	// To Change:
	EngineParameters.lookAheadTime = 250;
	EngineParameters.pGlobalSettingsBuffer = pGlobalSettingsData;
	EngineParameters.globalSettingsBufferSize = dwGlobalSettingsFileSize;
	EngineParameters.globalSettingsFlags = XACT_FLAG_GLOBAL_SETTINGS_MANAGEDATA;
	EngineParameters.fnNotificationCallback = XACTNotificationCallback;

	// Initialize the engine;
	if ( FAILED ( hr = m_pXACT3Engine->Initialize( &EngineParameters ) ) ) 
		return hr;

	// Register for Notifications

	// The "wave bank prepared" notification will let the app know when it is save to use
	// play cues that reference streaming wave data.
	XACT_NOTIFICATION_DESCRIPTION desc = {0};
	desc.flags = XACT_FLAG_NOTIFICATION_PERSIST;
	desc.type = XACTNOTIFICATIONTYPE_WAVEBANKPREPARED;
	m_pXACT3Engine->RegisterNotification( &desc );

	// The "sound bank destroyed" notification will let the app know when it is save to use
	// play cues that reference streaming wave data.
	desc.flags = XACT_FLAG_NOTIFICATION_PERSIST;
	desc.type = XACTNOTIFICATIONTYPE_SOUNDBANKDESTROYED;
	m_pXACT3Engine->RegisterNotification( &desc );

	// The "cue stop" notification will let the app know when it a song stops so a new one 
	// can be played
	desc.flags = XACT_FLAG_NOTIFICATION_PERSIST;
	desc.type = XACTNOTIFICATIONTYPE_CUESTOP;
	desc.cueIndex = XACTINDEX_INVALID;
	m_pXACT3Engine->RegisterNotification( &desc );

	// The "cue prepared" notification will let the app know when it a a cue that uses 
	// streaming data has been prepared so it is ready to be used for zero latency streaming
	desc.flags = XACT_FLAG_NOTIFICATION_PERSIST;
	desc.type = XACTNOTIFICATIONTYPE_CUEPREPARED;
	desc.cueIndex = XACTINDEX_INVALID;
	m_pXACT3Engine->RegisterNotification( &desc );

	// Find the named wave bank file
	hFile = CreateFile( "Resources\\Audio\\Win\\BasicWaveBank.xwb", GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL );

	// Use handle to create an "in memory" XACT wave bank file using memory mapped File IO
	// Memory mapped files are fastest assuming user has enough virtual address space for a full map
	if( hFile != INVALID_HANDLE_VALUE )
	{
		dwFileSize = GetFileSize( hFile, NULL );
		if( dwFileSize != -1 )
		{
			hMapFile = CreateFileMapping( hFile, NULL, PAGE_READONLY, 0, dwFileSize, NULL );
			if( hMapFile )
			{
				m_pbInMemoryWaveBank = MapViewOfFile( hMapFile, FILE_MAP_READ, 0, 0, 0 );
				if( m_pbInMemoryWaveBank )
				{
					hr = m_pXACT3Engine->CreateInMemoryWaveBank( m_pbInMemoryWaveBank, dwFileSize, 0,
						0, &m_pInMemoryWaveBank );
				}
				CloseHandle( hMapFile ); // pbWaveBank maintains a handle on the file so close this unneeded handle
			}
		}
		CloseHandle( hFile ); // pbWaveBank maintains a handle on the file so close this unneeded handle
	}

	if( FAILED ( hr ) )
		return E_FAIL;

	m_hStreamingWaveBank = CreateFile( "Resources\\Audio\\Win\\BasicStreamWaveBank.xwb",
		GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING,
		FILE_FLAG_OVERLAPPED | FILE_FLAG_NO_BUFFERING, NULL );

	if( m_hStreamingWaveBank != INVALID_HANDLE_VALUE )
	{
		XACT_WAVEBANK_STREAMING_PARAMETERS wsParams;
		ZeroMemory( &wsParams, sizeof( XACT_WAVEBANK_STREAMING_PARAMETERS ) );
		wsParams.file = m_hStreamingWaveBank;
		wsParams.offset = 0;

		// 64 means to allocate a 64 * 2k buffer for streaming.  
		// This is a good size for DVD streaming and takes good advantage of the read ahead cache
		wsParams.packetSize = 64;

		hr = m_pXACT3Engine->CreateStreamingWaveBank( &wsParams, &m_pStreamingWaveBank );
	}

	if( FAILED( hr ) )
		return E_FAIL; // CleanupXACT() will cleanup state before exiting


	// Find the named sound bank file
	hFile = CreateFile( "Resources\\Audio\\Win\\BasicSoundBank.xsb", GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL );

	// Read and Register sound bank with XACT.  No need for memory mapping.
	if( hFile != INVALID_HANDLE_VALUE )
	{
		dwFileSize = GetFileSize( hFile, NULL );
		if( dwFileSize != -1 )
		{
			// Allocate the data here and free the data when recieving the sound bank destroyed notification
			m_pbSoundBank = new BYTE[dwFileSize];
			if( m_pbSoundBank )
			{
				if( 0 != ReadFile( hFile, m_pbSoundBank, dwFileSize, &dwBytesRead, NULL ) )
				{
					hr = m_pXACT3Engine->CreateSoundBank( m_pbSoundBank, dwFileSize, 0,
						0, &m_pSoundBank );
				}
			}
		}
		CloseHandle( hFile );
	}

	if( FAILED ( hr ) )
		return E_FAIL;

	m_iBackgroundMusic = m_pSoundBank->GetCueIndex("ExceederBG");

	// Made it to end, return no errors
	m_bInitialized = true;
	return S_OK;
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		Update
//   Last Modified:	01/28/2009
//   Description:	Handle Sound updates and run the XACT Engine
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::Update()
{
	// HANDLE NOTIFICATIONS

	// Handle these notifications outside of the callback because
	// only a subset of XACT APIs can be called inside the callback.

	if( m_bStreamingWaveBankPrepared )
	{
		m_bStreamingWaveBankPrepared = false;

		// Starting playing background music after the streaming wave bank 
		// has been prepared but no sooner.  The background music does not need to be 
		// zero-latency so the cues do not need to be prepared first 
		m_pSoundBank->Play( m_iBackgroundMusic, 0, 0, NULL );
	}

	if( m_bZeroLatencyCueStop )
	{
		m_bZeroLatencyCueStop = false;

		m_pZeroLatencyRevCue->Destroy();
		m_pZeroLatencyRevCue = NULL;
	}

	if( m_bSongStopped )
	{
		m_bSongStopped = false;

		m_pSoundBank->Play( m_iBackgroundMusic, 0, 0, NULL );
	}


	// Has to be called to make any exact actions work
	if(m_pXACT3Engine)
		m_pXACT3Engine->DoWork();
	//Sleep(1);
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		PauseFX
//   Last Modified:	01/28/2009
//   Description:	Pause all sound effects
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::PauseFX()
{
	if (m_pXACT3Engine)
	{
		m_pXACT3Engine->Pause(m_pXACT3Engine->GetCategory("FX"), true);
		m_pXACT3Engine->Pause(m_pXACT3Engine->GetCategory("Default"), true);
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		PauseMusic
//   Last Modified:	01/28/2009
//   Description:	Pause the streaming music
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::PauseMusic()
{
	if (m_pXACT3Engine)
	{
		m_pXACT3Engine->Pause(m_pXACT3Engine->GetCategory("Music"), true);
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		PauseAll
//   Last Modified:	01/28/2009
//   Description:	Pause both the music and the sfx
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::PauseAll()
{
	PauseFX();
	PauseMusic();
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		UnPauseFX
//   Last Modified:	01/28/2009
//   Description:	Restart playback of sfx
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::UnPauseFX()
{
	if (m_pXACT3Engine)
	{
		m_pXACT3Engine->Pause(m_pXACT3Engine->GetCategory("FX"), false);
		m_pXACT3Engine->Pause(m_pXACT3Engine->GetCategory("Default"), false);
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		UnPauseMusic
//   Last Modified:	01/28/2009
//   Description:	Restart playback of music
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::UnPauseMusic()
{
	if (m_pXACT3Engine)
	{
		m_pXACT3Engine->Pause(m_pXACT3Engine->GetCategory("Music"), false);
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		UnPauseAll
//   Last Modified:	01/28/2009
//   Description:	Restart playback of sfx and music
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::UnPauseAll()
{
	UnPauseFX();
	UnPauseMusic();
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		ShutDown
//   Last Modified:	01/28/2009
//   Description:	Shut down the Sound Manager
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::ShutDown()
{
	if( !m_bInitialized)
		return;

	if(m_pXACT3Engine)
	{
		m_pXACT3Engine->ShutDown();
		m_pXACT3Engine->Release();
		m_pXACT3Engine = NULL;
	}

	// After pEngine->ShutDown() returns, it is safe to release audio file memory
	if( m_hStreamingWaveBank != INVALID_HANDLE_VALUE && m_hStreamingWaveBank != NULL )
		CloseHandle( m_hStreamingWaveBank );

	if(m_pbSoundBank)
		delete [] m_pbSoundBank;

	m_pSoundBank = NULL;

	if(m_pbInMemoryWaveBank)
		UnmapViewOfFile(m_pbInMemoryWaveBank);
	m_pbInMemoryWaveBank = NULL;

	CoUninitialize();
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		Play
//   Last Modified:	01/28/2009
//   Description:	Play the sound indicated by soundName
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::Play(PCSTR soundName)
{
	XACTINDEX index = m_pSoundBank->GetCueIndex(soundName);
	m_pSoundBank->Play(index, 0, 0, NULL);
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		PlayBuffered
//   Last Modified:	01/28/2009
//   Description:	Play the sound indicated by soundName only if there are
//					no instances currently playing
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::PlayBuffered(PCSTR soundName)
{
	XACTINDEX index = m_pSoundBank->GetCueIndex(soundName);
	XACT_CUE_PROPERTIES properties;
	m_pSoundBank->GetCueProperties(index, &properties);
	XACTINSTANCELIMIT b = properties.currentInstances;
	if (b <= 0)
	{
		m_pSoundBank->Play(index, 0, 0, NULL);
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		PlayStreaming
//   Last Modified:	01/28/2009
//   Description:	Play the streaming music indicated by soundName
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::PlayStreaming(PCSTR soundName)
{
	m_pSoundBank->Stop(m_iBackgroundMusic, 0);
	m_iBackgroundMusic = m_pSoundBank->GetCueIndex(soundName);
	m_pSoundBank->Play(m_iBackgroundMusic, 0, 0, NULL);
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		PlayStreamingBuffered
//   Last Modified:	01/28/2009
//   Description:	Play the streaming music indicated by soundName
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::PlayStreamingBuffered(PCSTR soundName)
{
	XACTINDEX index = m_pSoundBank->GetCueIndex(soundName);
	XACT_CUE_PROPERTIES properties;
	m_pSoundBank->GetCueProperties(index, &properties);
	XACTINSTANCELIMIT b = properties.currentInstances;
	if (b <= 0)
	{
		m_pSoundBank->Stop(m_iBackgroundMusic, 0);
		m_iBackgroundMusic = m_pSoundBank->GetCueIndex(soundName);
		m_pSoundBank->Play(m_iBackgroundMusic, 0, 0, NULL);
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		XACTNotificationCallback
//   Last Modified:	01/28/2009
//   Description:	Stop playback of the sound indicated by soundName
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::Stop(PCSTR soundName)
{
	XACTINDEX index = m_pSoundBank->GetCueIndex(soundName);
	m_pSoundBank->Stop(index, 0);
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		StopStreaming
//   Last Modified:	01/28/2009
//   Description:	Stop playback of streaming sound indicated by soundName
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::StopStreaming(PCSTR soundName)
{
	XACTINDEX index = m_pSoundBank->GetCueIndex(soundName);
	m_pSoundBank->Stop(index, 0);
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		GetSoundIndex
//   Last Modified:	01/28/2009
//   Description:	Return the sound index of the sound indicated by soundName
//
//////////////////////////////////////////////////////////////////////////////////

int CSoundManager::GetSoundIndex(PCSTR soundName)
{
	return m_pSoundBank->GetCueIndex(soundName);
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		SetVariable
//   Last Modified:	01/28/2009
//   Description:	Set the variable indicated by variableName to the value 
//					indicated by setValue in the XACT Engine
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::SetVariable(PCSTR variableName, float setValue)
{
	XACTINDEX index = m_pXACT3Engine->GetGlobalVariableIndex(variableName);
	m_pXACT3Engine->SetGlobalVariable(index, setValue);
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		SetMusicVolume
//   Last Modified:	01/28/2009
//   Description:	Set the volume of the music
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::SetMusicVolume(float volume)
{
	m_fMusicVolume = volume;
	if( m_bInitialized && m_pXACT3Engine )
	{
		m_pXACT3Engine->SetVolume(m_pXACT3Engine->GetCategory("FX"), m_fMusicVolume);
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		SetFXVolume
//   Last Modified:	01/28/2009
//   Description:	Set the volume of the sfx
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::SetFXVolume(float volume)
{
	m_fFXVolume = volume;
	if( m_bInitialized && m_pXACT3Engine )
	{
		m_pXACT3Engine->SetVolume(m_pXACT3Engine->GetCategory("FX"), m_fFXVolume);
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		IncreaseMusicVolume
//   Last Modified:	01/28/2009
//   Description:	Slightly increase the volume of the music
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::IncreaseMusicVolume()
{
	m_fMusicVolume += 10.0f * GameManager().Timer().GetDeltaTime();
	
	if (m_fMusicVolume > 6.0f)
	{
		m_fMusicVolume = 6.0f;
	}

	if( m_bInitialized && m_pXACT3Engine )
	{
		m_pXACT3Engine->SetVolume(m_pXACT3Engine->GetCategory("Music"), m_fMusicVolume);
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		DecreaseMusicVolume
//   Last Modified:	01/28/2009
//   Description:	Slightly decrease the volume of the music
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::DecreaseMusicVolume()
{
	m_fMusicVolume -= 10.0f * GameManager().Timer().GetDeltaTime();
	
	if (m_fMusicVolume < 0.0f )
	{
		m_fMusicVolume = 0.0f;
	}
	
	if( m_bInitialized && m_pXACT3Engine )
	{
		m_pXACT3Engine->SetVolume(m_pXACT3Engine->GetCategory("Music"), m_fMusicVolume);
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		IncreaseFXVolume
//   Last Modified:	01/28/2009
//   Description:	Slightly increase the volume of the sfx
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::IncreaseFXVolume()
{
	m_fFXVolume += 10.0f * GameManager().Timer().GetDeltaTime();
	
	if (m_fFXVolume > 6.0f)
	{
		m_fFXVolume = 6.0f;
	}

	if( m_bInitialized && m_pXACT3Engine )
	{
		m_pXACT3Engine->SetVolume(m_pXACT3Engine->GetCategory("FX"), m_fFXVolume);
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
//   Function:		DecreaseFXVolume
//   Last Modified:	01/28/2009
//   Description:	Slightly decrease the volume of the sfx
//
//////////////////////////////////////////////////////////////////////////////////

void CSoundManager::DecreaseFXVolume()
{
	m_fFXVolume -= 10.0f * GameManager().Timer().GetDeltaTime();

	if (m_fFXVolume < 0.0f )
	{
		m_fFXVolume = 0.0f;
	}

	if( m_bInitialized && m_pXACT3Engine )
	{
		m_pXACT3Engine->SetVolume(m_pXACT3Engine->GetCategory("FX"), m_fFXVolume);
	}
}

//-----------------------------------------------------------------------------------------
// This is the callback for handling XACT notifications.  This callback can be executed on a 
// different thread than the app thread so shared data must be thread safe.  The game 
// also needs to minimize the amount of time spent in this callbacks to avoid glitching, 
// and a limited subset of XACT API can be called from inside the callback so 
// it is sometimes necessary to handle the notification outside of this callback.
//-----------------------------------------------------------------------------------------
void WINAPI CSoundManager::XACTNotifyCallback( const XACT_NOTIFICATION* pNotification )
{
	// Need to put in the cue index for what has stopped 
	// &&  pNotification->cue.cueIndex == m_iBackgroundMusic 
	if( pNotification->type == XACTNOTIFICATIONTYPE_CUESTOP &&  pNotification->cue.cueIndex == m_iBackgroundMusic )
	{
		// What to do when an we receive a cue stop for the background music

		// Here we just notify the update that the music has stopped
		// should restart upon update
		m_bSongStopped = true;
	}

	if( pNotification->type == XACTNOTIFICATIONTYPE_WAVEBANKPREPARED &&
		pNotification->waveBank.pWaveBank == m_pStreamingWaveBank )
	{
		// Respond to this notification outside of this callback so Prepare() can be called
		m_bStreamingWaveBankPrepared = true;
	}

	if( pNotification->type == XACTNOTIFICATIONTYPE_SOUNDBANKDESTROYED &&
		pNotification->soundBank.pSoundBank == m_pSoundBank )
	{
		// Cleanup sound bank memory
		if( m_pbSoundBank )
		{
			delete[] m_pbSoundBank;
			m_pbSoundBank = NULL;
		}
	}

	if( pNotification->type == XACTNOTIFICATIONTYPE_CUESTOP &&
		pNotification->cue.pCue == m_pZeroLatencyRevCue)
	{
		// Respond to this notification outside of this callback so Prepare() can be called
		m_bZeroLatencyCueStop = true;
	}
}